from enum import Enum

# noinspection PyArgumentList
TYPE = Enum('TYPE', 'stimulus measurement scan_config analysis emka', module=__name__)

FILE_TYPES = {
    TYPE.stimulus: ('.mat', [('matlab', '.mat'), ('json', '.json'), ('python', '.pkl')]),
    TYPE.measurement: ('.csv', [('table', '.csv'), ('matlab', '.mat'), ('python', '.pkl')]),
    TYPE.scan_config: ('.cfg', [('scanImage', '.cfg')]),
    TYPE.analysis: ('.csv', [('data frame', '.pandas'), ('table', '.csv'), ('python', '.pkl')]),
    TYPE.emka: ('.raw', [('raw file', '.raw'), ('raw file txt', '.txt')])}



from enum import Enum
from os import getcwd
from os.path import expanduser
from typing import Union, List, Iterable

try:
    # noinspection PyPackageRequirements
    import wx
    # noinspection PyPackageRequirements
    from wx.lib.agw import multidirdialog as mdd
except ImportError:
    wx, mdd = None, None
    raise ImportError('install wx to use wx ui functions')

# noinspection PyArgumentList
DialogStyle = Enum('DialogStyle', 'open save multiple')

_file_dialog_style = {DialogStyle.open: wx.FD_OPEN | wx.FD_FILE_MUST_EXIST,
                      DialogStyle.save: wx.FD_SAVE,
                      DialogStyle.multiple: wx.FD_OPEN | wx.FD_MULTIPLE | wx.FD_FILE_MUST_EXIST}


def _select_item(dialog, style: DialogStyle) -> Union[str, List[str]]:
    dialog.SetSize(0, 0, 200, 50)
    result = dialog.ShowModal()
    if result == wx.ID_CANCEL:
        raise KeyboardInterrupt
    if style == DialogStyle.multiple:
        chosen_path = dialog.GetPaths()
    else:
        chosen_path = dialog.GetPath() if hasattr(dialog, 'GetPath') else dialog.GetPaths()[0]
    dialog.Destroy()
    return chosen_path


def file_to_open(ext: Union[str, List[str]] = "", default_dir: str = getcwd(),
                 style: DialogStyle = DialogStyle.open) -> Union[str, List[str]]:
    if isinstance(ext, str):
        wildcard = "{0}|*{1}".format(ext[1:], ext)  # no space between pipe and wildcard
    elif isinstance(ext, list) or isinstance(ext, tuple):  # Default (*) | * would not give anything
        wildcard = "{0}|{1}".format(ext[0][1:], ';'.join('*' + x for x in ext))
    else:
        raise ValueError('file filter error: ' + str(ext))
    _ = wx.App()
    open_file_dialog = wx.FileDialog(None, "Select File", defaultDir=default_dir, wildcard=wildcard,
                                     style=_file_dialog_style[style])
    return _select_item(open_file_dialog, style)


def files_to_open(ext: Union[str, List[str]] = "", default_dir: str = getcwd()) -> List[str]:
    return file_to_open(ext, default_dir, DialogStyle.multiple)


def file_to_save(ext: Union[str, List[str]] = "", default_dir: str = getcwd()) -> str:
    return file_to_open(ext, default_dir, DialogStyle.save)


_folder_dialog_style = {DialogStyle.open: mdd.DD_DIR_MUST_EXIST,
                        DialogStyle.save: mdd.DD_NEW_DIR_BUTTON,
                        DialogStyle.multiple: mdd.DD_DIR_MUST_EXIST | mdd.DD_MULTIPLE}


def _folder_dialog_filter(folder: str) -> str:
    return expanduser('~' + folder[14:]) if folder.startswith('Home directory') else folder


def folder_to_open(default_dir: str = getcwd(), style: DialogStyle = DialogStyle.open) -> Union[str, Iterable[str]]:
    _ = wx.App()
    open_folder_dialog = wx.lib.agw.multidirdialog.MultiDirDialog(
        None, "Select Folder", defaultPath=default_dir, agwStyle=_folder_dialog_style[style])
    selection = _select_item(open_folder_dialog, style)
    if isinstance(selection, str):
        return _folder_dialog_filter(selection)
    elif isinstance(selection, Iterable):
        return (_folder_dialog_filter(x) for x in selection)


def folders_to_open(default_dir: str = getcwd()) -> Iterable[str]:
    return folder_to_open(default_dir, style=DialogStyle.multiple)


def folder_to_save(default_dir: str = getcwd()) -> str:
    return folder_to_open(default_dir, style=DialogStyle.save)



from setuptools import setup

setup(
    name='uifunc',
    version='0.1',
    packages=['uifunc'],
    url='https://github.com/Palpatineli/uifunc',
    download_url='https://github.com/Palpatineli/uifunc/archive/0.1.tar.gz',
    license='GPLv3',
    author='Keji Li',
    author_email='mail@keji.li',
    extra_require={'wx': ['wx']},
    description='convenience functions for opening and saving files/folders',
    classifiers=['Development Status :: 4 - Beta',
                 'Programming Language :: Python :: 3']
)



# provide decorators for functions that reads a folder/file name/list as the only argument
import sys
from typing import Callable, Any, Optional

try:
    from .wxopen import *
except ImportError:
    from .tkopen import *


class FolderSelector(object):
    def __init__(self, func: Callable[[str], Any]):
        self.func = func

    def __call__(self)-> Callable[[], Any]:
        if len(sys.argv) == 2:
            folder = sys.argv[1]
        else:
            folder = folder_to_open()
        return self.func(folder)


class FoldersSelector(object):
    def __init__(self, func: Callable[[List[str]], Any]):
        print(type(func), func)
        self.func = func

    def __call__(self) -> Callable[[], Any]:
        if len(sys.argv) > 1:
            folders = sys.argv[1:]
        else:
            folders = folders_to_open()
        return self.func(folders)


class FilesSelector(object):
    def __init__(self, files: Optional[List[str]]=None):
        self.files = files

    def __call__(self, func: Callable[[List[str]], Any]) -> Callable[[], Any]:
        files = self.files
        if len(sys.argv) > 1:
            files = sys.argv[1:]
        elif files and files[0].startswith('.'):
            files = files_to_open(files)
        elif files is None:
            raise ValueError("supply a list of files or file filters in argument or command line arguments")
        return func(files)


class FileSelector(object):
    def __init__(self, file: Optional[List[str]]=None):
        self.file = file

    def __call__(self, func: Callable[[str], Any]) -> Callable[[], Any]:
        file = self.file
        if len(sys.argv) == 2:
            file = sys.argv[1]
        elif file and file[0].startswith('.'):
            file = file_to_open(file)
        elif file is None:
            raise ValueError("supply a list of files or file filters in argument or command line arguments")
        return func(file)



"""convenience functions for choosing file with Tk"""
# noinspection SpellCheckingInspection
import tkinter.filedialog as tkfd
from os import path, getcwd
from tkinter import Tk
from typing import Union, List, Iterable

from .common_file_types import TYPE, FILE_TYPES


def _choose_file(file_type: Union[TYPE, str, List[str]], func) -> Union[str, List[str]]:
    root = Tk()
    if isinstance(file_type, TYPE):
        default_ext, file_ext_list = FILE_TYPES[file_type]
        file_type_list = tuple((x[1:], '*' + x) for x in file_ext_list)
    elif isinstance(file_type, str):
        default_ext, file_type_list = file_type, ((file_type[1:], '*' + file_type),)
    elif isinstance(file_type, Iterable):
        default_ext, file_type_list = next(iter(file_type))[1:], tuple((x[1:], '*' + x) for x in file_type)
    else:
        raise ValueError('wrong file type :' + str(file_type))
    file_name = func(parent=root, defaultextension=default_ext, filetypes=file_type_list, initialdir=getcwd())
    root.destroy()
    return file_name


def file_to_open(file_type: Union[TYPE, str]) -> str:
    return _choose_file(file_type, tkfd.askopenfilename)


def files_to_open(file_type: Union[TYPE, str]) -> List[str]:
    return _choose_file(file_type, tkfd.askopenfilenames)


def file_to_save(file_type: Union[TYPE, str]) -> str:
    return _choose_file(file_type, tkfd.asksaveasfilename)


def folder_to_open(default_dir: str = getcwd()) -> str:
    root = Tk()
    folder = tkfd.askdirectory(parent=root, initialdir=default_dir)
    root.destroy()
    return folder


folder_to_save = folder_to_open


def swap_ext(file_name: str, ext: str) -> str:
    return path.splitext(file_name)[0] + ext
